package gridbus.gmd.security;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Title:        The Grid Market Directory (GMD) Toolkit
 * Developer:    The Gridbus Project
 *               The Grid Computing and Distributed Systems (GRIDS) Laboratory
 *               The University of Melbourne, Australia
 * URL:          http://www.gridbus.org
 * @version      Alpha Version,
 * Description:  The GMD Toolkit allows the creation of portal for management of
 *               Grid Market services. It also provide webservices based access
 *               to GMD from external applications.
 * @author       Jia Yu (program developer) and Rajkumar Buyya (project director)
 *
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */
public class JSPHandler{
	protected HttpServletRequest request;
	protected HttpServletResponse response;
	protected ServletContext context;
	
	protected SessionWrapper sw=null;
	
	protected boolean requiresLoggedIn;
	protected boolean requiresLoggedOut;
	
		/**
		 * Method process.
		 * @param req
		 * @param resp
		 * @param c
		 */
		public void process(HttpServletRequest req, HttpServletResponse resp, ServletContext c){
			request=req;
			response=resp;
			context=c;
		}
		
		/**
		 * Method redirect.
		 * @param url
		 * @return boolean
		 */
		public boolean redirect(String url){
			try{
					response.sendRedirect(url);
			}catch(IOException e){
				return false;
			}
			return true;
		}
		
		/**
		 * Method forward.
		 * @param url
		 * @return boolean
		 */
		public boolean forward(String url){
			try{
				RequestDispatcher rd=context.getRequestDispatcher(url);
				System.out.println(context);
				System.out.println(rd);
				if(rd!=null){
					rd.forward(request,response);
					System.out.println("forward");
				}
			}catch(IOException e){
				System.err.println(e.getMessage());
			}catch(ServletException e){
				System.err.println(e.getMessage());
			}
			return true;
		}
	
		/**
		 * Method loginStateInvalid.
		 * @return boolean
		 */
		public boolean loginStateInvalid(){
			boolean isLoggedIn=isLoggedIn();
			if(isLoggedIn && requiresLoggedOut) return true;
			else if(!isLoggedIn && requiresLoggedIn) return true;
			else return false;
		}	
		
		/**
		 * Method isLoggedIn.
		 * @return boolean
		 */
		public boolean isLoggedIn(){
			return getSessionWrapper().isLoggedIn();
		}
		
		/**
		 * Method isPost.
		 * @return boolean
		 */
		public boolean isPost(){
			return request.getMethod().equals("POST");
		}
		/**
		 * Method getRequestURI.
		 * @return String
		 */
		protected String getRequestURI(){
			return request.getRequestURI();
		}
	
		/**
		 * Method getSessionWrapper.
		 * @return SessionWrapper
		 */
		protected SessionWrapper getSessionWrapper(){
			if(sw==null){
				synchronized(this){
						if(sw==null){
							sw=new SessionWrapper(request.getSession(true));
						}
				}
			}
			return sw;
		}
		
		
}
