/**
 * Title:        The Grid Market Directory (GMD) Toolkit
 * Developer:    The Gridbus Project
 *               The Grid Computing and Distributed Systems (GRIDS) Laboratory
 *               The University of Melbourne, Australia
 * URL:          http://www.gridbus.org
 * @version      Alpha Version,
 * Description:  The GMD Toolkit allows the creation of portal for management of
 *               Grid Market services. It also provide webservices based access
 *               to GMD from external applications.
 * @author       Jia Yu (program developer) and Rajkumar Buyya (project director)
 *
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 */
 
package gridbus.gmd.soap.gmdqueryclient;

import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Properties;
import java.util.Vector;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.StringReader;

import org.apache.soap.Fault;
import org.apache.soap.rpc.Call;
import org.apache.soap.rpc.Response;
import org.apache.soap.rpc.Parameter;
import org.jdom.JDOMException;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.XMLOutputter;

public class GMDQuery {
//    static String DEFAULT_SERVICE_POINT="http://jarrett.cs.mu.oz.au:8080/soap/servlet/rpcrouter";
	Call myrpc;
	String servicePoint;
	
	public GMDQuery() throws GMDQueryException{
		
//		servicePoint=DEFAULT_SERVICE_POINT;
		Properties props = new Properties();
		String pFile="GMDQuery.prop";
		try {
                props.load(new FileInputStream(pFile));
            } catch(IOException e) {
                throw new GMDQueryException("Failed to load in the property file: " + pFile);
            }
        Enumeration enum = props.propertyNames();
        while (enum.hasMoreElements()) {
             String key = (String)enum.nextElement();
             if (System.getProperty(key) == null) {
                // command line properties take precedence
             	System.setProperty(key, props.getProperty(key));
             }
        }
        servicePoint="http://"+System.getProperty("gridbus.gmd.GMDHostAddress")+":"+System.getProperty("gridbus.gmd.GMDPort")+System.getProperty("gridbus.gmd.SOAPDirectory");
        System.out.println("Current GMD is on "+servicePoint+"\n");
   
		myrpc = new Call();
		myrpc.setTargetObjectURI(System.getProperty("gridbus.gmd.SOAPQueryService"));
     }
	
	public GMDQuery(String host, int port, String soapDir, String soapServiceName) throws GMDQueryException{
//		servicePoint=DEFAULT_SERVICE_POINT;
		Properties props = new Properties();
            servicePoint="http://"+host+":"+String.valueOf(port)+soapDir;
        System.out.println("Current GMD is on "+servicePoint+"\n");
   
		myrpc = new Call();
		myrpc.setTargetObjectURI(soapServiceName);
     }

	
	/**
	 * Method getPrice.
	 * @param String providerName
	 * @return Price
	 * @throws GMDQueryException
	 */
	public Price getPrice(String sName) throws GMDQueryException{
		Element QueryPriceRoot=new Element("QUERY-PRICE");
		QueryPriceRoot.addContent(new Element("SERVICE_NAME").setText(sName));
		Document QueryPrice=new Document(QueryPriceRoot);
		XMLOutputter outputter=new XMLOutputter(" ",true);
		String outReq=outputter.outputString(QueryPrice);
		
		//for looking at xml message
		//System.out.println(outReq);
				
	    myrpc.setMethodName("queryPrice");
	    
	    String rxml=null;
	    try{
	    	rxml=executeSOAPQuery(outReq);
	    }catch(GMDQueryException e){
	    	throw e;
	    }
	    
	    //for looking at xml message 
	    //System.out.println(rxml);
	    
        SAXBuilder builder=new SAXBuilder();
		Document PriceDetail=null;
						
		try{
				PriceDetail=builder.build(new StringReader(rxml));
		}catch(JDOMException e){
				System.err.println("Request parsing error"+e.getMessage());
		}
					
		Element priceRoot=PriceDetail.getRootElement();
		
		if(priceRoot.getAttribute("STATUS").getValue().equals("ERROR")) 
			throw new GMDQueryException(priceRoot.getChild("REASON").getText());
		
	    float hPrice=Float.parseFloat(priceRoot.getChild("HARDWARE").getText());
	    float sPrice=Float.parseFloat(priceRoot.getChild("SOFTWARE").getText());
    
	    
	    return new Price(hPrice,sPrice);
	}
	
	public GMDService getServiceByHost(String hName) throws GMDQueryException{
		Element QueryServiceRoot=new Element("QUERY-HOST");
		QueryServiceRoot.addContent(new Element("HOST_NAME").setText(hName));
		Document QueryService=new Document(QueryServiceRoot);
		XMLOutputter outputter=new XMLOutputter(" ",true);
		String outReq=outputter.outputString(QueryService);
		
		//for looking at xml
		//System.out.println(outReq);
		
		
	    myrpc.setMethodName("queryServiceByHost");
	    
	    String rxml=null;
	    try{
	    	rxml=executeSOAPQuery(outReq);
	    }catch(GMDQueryException e){
	    	throw e;
	    }
	    
	    //for looking at xml
	    //System.out.println(rxml);
	    
	    
        SAXBuilder builder=new SAXBuilder();
		Document ServiceDetail=null;
						
		try{
				ServiceDetail=builder.build(new StringReader(rxml));
		}catch(JDOMException e){
				System.err.println("Request parsing error"+e.getMessage());
		}
					
		Element serviceRoot=ServiceDetail.getRootElement();
		
		if(serviceRoot.getAttribute("STATUS").getValue().equals("ERROR")) 
			throw new GMDQueryException(serviceRoot.getChild("REASON").getText());
		
		String sn=serviceRoot.getChild("NAME").getText();
		String st=serviceRoot.getChild("TYPE").getText();
        String pn=serviceRoot.getChild("PROVIDER").getText();
        float hPrice=Float.parseFloat(serviceRoot.getChild("HPRICE").getText());
	    float sPrice=Float.parseFloat(serviceRoot.getChild("SPRICE").getText());
    	String des=serviceRoot.getChild("DESCRIPTION").getText();
	    
	    return new GMDService(sn,st,pn,hName,hPrice,sPrice,des);
	}
	
	
	/**
	 * Method getProviderList.
	 * @param String serviceType
	 * @throws GMDQueryException
	 */
	public ListIterator getServiceContactList(String sType) throws GMDQueryException{
		
		Element QueryServiceContactRoot=new Element("QUERY-SERVICE-CONTACT");
		QueryServiceContactRoot.addContent(new Element("SERVICE_TYPE").setText(sType));
		Document QueryServiceContact=new Document(QueryServiceContactRoot);
		XMLOutputter outputter=new XMLOutputter(" ",true);
		String outReq=outputter.outputString(QueryServiceContact);
	    
	    //for looking at xml message
	    //System.out.println(outReq);
	    
	    myrpc.setMethodName("queryServiceContact");
	    String rxml=null;
	    try{
	    	rxml=executeSOAPQuery(outReq);
	    }catch(GMDQueryException e){
			throw e;    
	    }
		//for looking at xml message
		//System.out.println(rxml);	
		
		
		SAXBuilder builder=new SAXBuilder();
		Document Services=null;
		
		try{
			Services=builder.build(new StringReader(rxml));
		}catch(JDOMException e){
			throw new GMDQueryException("Parsing result error");
		}
	
		LinkedList sal=new LinkedList();
		
		Element servicesRoot=Services.getRootElement();
		
		if(servicesRoot.getAttribute("STATUS").getValue().equals("ERROR")) 
			throw new GMDQueryException(servicesRoot.getChild("REASON").getText());
				
		List children = servicesRoot.getChildren();
    	Iterator iterator = children.iterator();
    	while (iterator.hasNext()) {
      	Element service = (Element) iterator.next();
      		sal.add(new ServiceAddress(service.getChild("NAME").getText(),service.getChild("ADDRESS").getText()));
      	}
      	return sal.listIterator();
	}
	
	
		
	/**
	 * Method getServiceList.
	 * @param sType
	 * @return ListIterator
	 * @throws GMDQueryException
	 */
	public ListIterator getServiceListByType(String sType) throws GMDQueryException{
		
		Element QueryServiceRoot=new Element("QUERY-SERVICE");
		QueryServiceRoot.addContent(new Element("SERVICE_TYPE").setText(sType));
		Document QueryService=new Document(QueryServiceRoot);
		XMLOutputter outputter=new XMLOutputter(" ",true);
		String outReq=outputter.outputString(QueryService);
		
		//for looking at xml message
		//System.out.println(outReq);
	    
	    myrpc.setMethodName("queryServiceByType");
	    String rxml=null;
	    try{
	    	rxml=executeSOAPQuery(outReq);
	    }catch(GMDQueryException e){
			throw e;    
	    }
		
		//for looking at xml message
		//System.out.println(rxml);
				
		SAXBuilder builder=new SAXBuilder();
		Document Services=null;
		
		try{
			Services=builder.build(new StringReader(rxml));
		}catch(JDOMException e){
			throw new GMDQueryException("Parsing result error");
		}
	
		LinkedList sl=new LinkedList();
		
		Element servicesRoot=Services.getRootElement();
		
		if(servicesRoot.getAttribute("STATUS").getValue().equals("ERROR")) 
			throw new GMDQueryException(servicesRoot.getChild("REASON").getText());
				
		List children = servicesRoot.getChildren();
    	Iterator iterator = children.iterator();
    	while (iterator.hasNext()) {
      	Element service = (Element) iterator.next();
      		sl.add(new GMDService(service.getChild("NAME").getText(),sType, service.getChild("PROVIDER").getText(),
      		service.getChild("ADDRESS").getText(),Float.parseFloat(service.getChild("PRICE").getChild("HARDWARE").getText()),	
      		Float.parseFloat(service.getChild("PRICE").getChild("SOFTWARE").getText()), service.getChild("DESCRIPTION").getText()));
      	}
      	return sl.listIterator();
	}
	
	public ListIterator getServiceListByProvider(String pName) throws GMDQueryException{
		Element QueryServiceRoot=new Element("QUERY-SERVICE");
		QueryServiceRoot.addContent(new Element("PROVIDER_NAME").setText(pName));
		Document QueryService=new Document(QueryServiceRoot);
		XMLOutputter outputter=new XMLOutputter(" ",true);
		String outReq=outputter.outputString(QueryService);
		
		//for looking at xml message
		//System.out.println(outReq);
	    
	    myrpc.setMethodName("queryServiceByProvider");
	    String rxml=null;
	    try{
	    	rxml=executeSOAPQuery(outReq);
	    }catch(GMDQueryException e){
			throw e;    
	    }
		
		//for looking at xml message
		//System.out.println(rxml);
				
		SAXBuilder builder=new SAXBuilder();
		Document Services=null;
		
		try{
			Services=builder.build(new StringReader(rxml));
		}catch(JDOMException e){
			throw new GMDQueryException("Parsing result error");
		}
	
		LinkedList sl=new LinkedList();
		
		Element servicesRoot=Services.getRootElement();
		
		if(servicesRoot.getAttribute("STATUS").getValue().equals("ERROR")) 
			throw new GMDQueryException(servicesRoot.getChild("REASON").getText());
				
		List children = servicesRoot.getChildren();
    	Iterator iterator = children.iterator();
    	while (iterator.hasNext()) {
      	Element service = (Element) iterator.next();
      		sl.add(new GMDService(service.getChild("NAME").getText(),service.getChild("TYPE").getText(), pName,
      		service.getChild("ADDRESS").getText(),Float.parseFloat(service.getChild("PRICE").getChild("HARDWARE").getText()),	
      		Float.parseFloat(service.getChild("PRICE").getChild("SOFTWARE").getText()), service.getChild("DESCRIPTION").getText()));
      	}
      	return sl.listIterator();
	
	}
	
	/**
	 * Method getServiceList.
	 * @param sType
	 * @return ListIterator
	 * @throws GMDQueryException
	 */
	public ListIterator getServiceList() throws GMDQueryException{
		
				
		//for looking at xml message
		//System.out.println(outReq);
	    
	    myrpc.setMethodName("queryService");
	    String rxml=null;
	    try{
	    	rxml=executeSOAPQuery("");
	    }catch(GMDQueryException e){
			throw e;    
	    }
		
		//for looking at xml message
		//System.out.println(rxml);
				
		SAXBuilder builder=new SAXBuilder();
		Document Services=null;
		
		try{
			Services=builder.build(new StringReader(rxml));
		}catch(JDOMException e){
			throw new GMDQueryException("Parsing result error");
		}
	
		LinkedList sl=new LinkedList();
		
		Element servicesRoot=Services.getRootElement();
		
		if(servicesRoot.getAttribute("STATUS").getValue().equals("ERROR")) 
			throw new GMDQueryException(servicesRoot.getChild("REASON").getText());
				
		List children = servicesRoot.getChildren();
    	Iterator iterator = children.iterator();
    	while (iterator.hasNext()) {
      	Element service = (Element) iterator.next();
      		sl.add(new GMDService(service.getChild("NAME").getText(),service.getChild("TYPE").getText(), service.getChild("PROVIDER").getText(),
      		service.getChild("ADDRESS").getText(),Float.parseFloat(service.getChild("PRICE").getChild("HARDWARE").getText()),	
      		Float.parseFloat(service.getChild("PRICE").getChild("SOFTWARE").getText()), service.getChild("DESCRIPTION").getText()));
      	}
      	return sl.listIterator();
	}
	
	
	private String executeSOAPQuery(String outReq) throws GMDQueryException{
	
		myrpc.setEncodingStyleURI("http://schemas.xmlsoap.org/soap/encoding/");
		
		//Create a URL object, which represents the endpoint
		URL url=null;
		
		try{
			url=new URL(servicePoint);
		}catch(java.net.MalformedURLException e){
			throw new GMDQueryException(e.getMessage());
		}	
		
		// Add Parameters
	    if(!outReq.equals("")){ 
	   		Vector myparams = new Vector(1);
			myparams.addElement(new Parameter("inputReq", String.class, outReq,null));
			myrpc.setParams(myparams);
	    }
		
		//Send the SOAP RPC request message using invoke() method
		Response resp=null;
		
		try{
			resp=myrpc.invoke(url," "); 
		}catch(org.apache.soap.SOAPException e){
			throw new GMDQueryException("connection error");
		}	
		
		//Check the response.
		Parameter result=null;
		
		if(resp.generatedFault()){    //Error Occured
			Fault fault=resp.getFault();
			throw new GMDQueryException("GMD Error:"+fault.getFaultString());
		}else {
			 result=resp.getReturnValue();
		}
		return result.getValue().toString();
	}
	
}
